#include <iostream>
#include <wstrtest.hh>

using namespace std;

//
// Example class implementing IDL interface wst::Iface
//
class Iface_i: public POA_wst::Iface,
                public PortableServer::RefCountServantBase {
private:
  // Make sure all instances are built on the heap by making the
  // destructor non-public
  //virtual ~Iface_i();
public:
  // standard constructor
  Iface_i();
  virtual ~Iface_i();

  // methods corresponding to defined IDL attributes and operations
  void foo(const wst::MyStruct& s);

};

//
// Example implementational code for IDL interface wst::Iface
//
Iface_i::Iface_i(){
  // add extra constructor code here
}
Iface_i::~Iface_i(){
  // add extra destructor code here
}
//   Methods corresponding to IDL attributes and operations
void Iface_i::foo(const wst::MyStruct& s){
	std::wcout << L"Received MyStruct:" << std::endl
		<< L"MyStruct::a = " << static_cast<const wchar_t*>(s.a) << std::endl
		<< L"MyStruct::b = " << static_cast<const wchar_t*>(s.b) << std::endl;
}



// End of example implementational code



int main(int argc, char** argv)
{
  try {
    // Initialise the ORB.
    CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);

    // Obtain a reference to the root POA.
    CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
    PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

    // We allocate the objects on the heap.  Since these are reference
    // counted objects, they will be deleted by the POA when they are no
    // longer needed.
    Iface_i* myIface_i = new Iface_i();


    // Activate the objects.  This tells the POA that the objects are
    // ready to accept requests.
    PortableServer::ObjectId_var myIface_iid = poa->activate_object(myIface_i);


    // Obtain a reference to each object and output the stringified
    // IOR to stdout
    {
      // IDL interface: wst::Iface
      CORBA::Object_var ref = myIface_i->_this();
      CORBA::String_var sior(orb->object_to_string(ref));
      cout << "IDL object wst::Iface IOR = '" << (char*)sior << "'" << endl;
    }



    // Obtain a POAManager, and tell the POA to start accepting
    // requests on its objects.
    PortableServer::POAManager_var pman = poa->the_POAManager();
    pman->activate();

    orb->run();
    orb->destroy();
  }
  catch(CORBA::SystemException&) {
    cerr << "Caught CORBA::SystemException." << endl;
  }
  catch(CORBA::Exception&) {
    cerr << "Caught CORBA::Exception." << endl;
  }
  catch(omniORB::fatalException& fe) {
    cerr << "Caught omniORB::fatalException:" << endl;
    cerr << "  file: " << fe.file() << endl;
    cerr << "  line: " << fe.line() << endl;
    cerr << "  mesg: " << fe.errmsg() << endl;
  }
  catch(...) {
    cerr << "Caught unknown exception." << endl;
  }

  return 0;
}

